/******************************************************************************
*  Record of a class gained by a persona (XP, level, hit points, etc.).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-22
*  @version  1.0
******************************************************************************/

public class ClassRecord {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Class info for this class. */
	ClassInfo info;

	/** Level in this class. */
	int level;

	/** Hit points earned for this class. */
	int hitPoints;

	/** Experience points earned towards this class. */
	int XP;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (by ClassInfo, level, random hit points).
	*/
	public ClassRecord (ClassInfo info, int level, int hpBonus) {
		this.info = info;
		this.level = level;
		this.XP = info.getExperienceReq(level);
		rollFullHitPoints(hpBonus);
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------
	public ClassInfo getInfo () { return info; }
	public int getLevel () { return level; }
	public int getHitPoints () { return hitPoints; }
	public int getXP () { return XP; }

	/**
	*  Return the base attack bonus for this class/level.
	*/
	public int attackBonus () {
		return info.getAttackBonus(level);
	}

	/**
	*  Add XP to this class.
	*  @return Did we gain a new level?
	*/
	public boolean addXP (int awardXP) {
		XP += awardXP;
		if (XP >= info.getExperienceReq(level+1)) {
			level++;
			XP = Math.min(XP, info.getExperienceReq(level+1)-1);
			return true;
		}
		return false;
	}

	/**
	*  Add hit points for a newly-gained level.
	*/
	public void rollNewHitPoints (int hpBonus) {

		// Advance 0-level (normal man) to 1st
		if (level == 1) {
			hitPoints += (info.getHitDiceType() - 6)/2;
			hitPoints = Math.max(hitPoints, 1 + hpBonus);
		}
		
		// Levels above 1st
		else if (level > 1) {
			Dice newDice = info.getHitDiceInc(level);
			if (newDice.getNumber() > 0)
				hitPoints += newDice.rollPlus(hpBonus);
			else 
				hitPoints += newDice.getAddition();
		}		
	}

	/**
	*  Roll full hit points from start to current level.
	*/
	public void rollFullHitPoints (int hpBonus) {
		if (level <= 0) {
			hitPoints = new Dice(1, 6).rollPlus(hpBonus);
		}
		else {
			hitPoints = info.getHitDiceInc(1).rollPlus(hpBonus);
			int fullLevel = level;
			for (level = 2; level <= fullLevel; level++) {
				rollNewHitPoints(hpBonus);
			}
			level = fullLevel;
		}
	}

	/**
	*  Return a string representation of this class record.
	*/
	public String toString() {
		return info.getName() + "(" + level + ", " + hitPoints + ", " + XP + ")";
	}	

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
	}
}

