import java.util.ArrayList;
import java.text.DecimalFormat;

/******************************************************************************
*  Statistical bin for a given group of persona.
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-07-01
*  @version  1.02
******************************************************************************/

public class StatBin {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Array of personae included. */
	ArrayList<Persona> bin; 

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor.
	*/
	public StatBin () {
		bin = new ArrayList<Persona>();
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Get the total number of personas.
	*/
	public int getNum () { return bin.size(); }

	/**
	*  Square an integer.
	*/
	static long square (long i) {
		return i * i;	
	}

	/**
	*  Compute a mean from num & sum.
	*/
	double mean (int num, long sum) {
		return (double) sum/num;	
	}

	/**
	*  Compute a standard deviation from num, sum, and sum-of-squares.
	*/
	double stdev (int num, long sum, long sumSquare) {
		return Math.sqrt((sumSquare - (double) square(sum)/num)/num);
	}

	/**
	*  Add another persona into these stats.
	*/
	public void addPersona (Persona p) {
		bin.add(p);
	}

	/**
	*  Generate & print all the stats we have available.
	*/
	public void printAllStats () {
		int num = getNum();
		System.out.print(num + "\t");
		if (num < 1) {
			System.out.println("No data");
			return;
		}
		else {
		
			// Sum hit points & ability scores
			long sumHitPoints = 0;
			int numAbilities = Ability.values().length;
			long [] sumAbilityScore = new long[numAbilities];
			for (Persona p: bin) {
				sumHitPoints += p.getHitPoints();
		 		for (int i = 0; i < numAbilities; i++) {
					sumAbilityScore[i] += p.getAbilityScoreByIndex(i);
				}
			}

			// Print means
			DecimalFormat df = new DecimalFormat("0.0");
			System.out.print(Math.round(mean(num, sumHitPoints)) + "\t");
	 		for (int i = 0; i < sumAbilityScore.length; i++) {
				double mean = mean(num, sumAbilityScore[i]);
				System.out.print(df.format(mean) + "\t");
			}
 		}
		System.out.println("");
	}

	/**
	*  Generate & print all individual data we have available.
	*/
	public void printAllData () {
		for (Persona p: bin) {
			System.out.print(p.getHitPoints() + "\t");
	 		for (int i = 0; i < Ability.values().length; i++) {
				System.out.print(p.getAbilityScoreByIndex(i) + "\t");
			}
			System.out.println();
		}
	}

	/*--------------------------------------------------------------------------
	* NOTES:
	*-------------------------------------------------------------------------*/
}

